--GALPSP GFX Initialization v0.2
--This file initializes the Graphics for GALPSP Lib, please don't edit it.
--CHANGE LOG:
--0.1 - First release
--0.2 - Changed the way the Backgrounds are drawn, a little more processor intensive, but allows to do more stuff.
--		Changed galpsp_charin() to allow non animated entrances.
--		Added black color
--		Added new position variable

if sys_colors == true then
	white = Color.new(255,255,255)
	gray = Color.new(192,192,192)
	gray_deep = Color.new(128,128,128)
	red = Color.new(255,0,0)
	yellow = Color.new(255,255,0)
	green = Color.new(0,255,0)
	cyan = Color.new(0,255,255)
	blue = Color.new(0,0,255)
	magenta = Color.new(255,0,255)
	black = Color.new(0,0,0)
end

--Load User Data
dofile("./Data/Conf/characters.lua")
dofile("./Data/Conf/objects.lua")
dofile("./Data/Conf/bg.lua")

--Load System data
gui = Image.load("./Data/GFX/System/GUI.png")
blink_gfx = Image.load("./Data/GFX/System/pressa.png")

--Create System Placeholders
--//Character windows
leftw = Image.createEmpty(char_width,char_height)
rightw = Image.createEmpty(char_width,char_height)
centerw = Image.createEmpty(char_width,char_height)
--//BG Window
bg = Image.createEmpty(480,272)
--//Text Windows
namew = Image.createEmpty(namew_width,10)
textw = Image.createEmpty(textw_width,textw_height)
optionw = Image.createEmpty(416,66)
--//Debug Window (developer only)
debugw = Image.createEmpty(480,272)
--//Object Windows
animw = Image.createEmpty(480,272)
blink = Image.createEmpty(20,20)

--Variables
--//Screen Positions
leftx = leftf --Current left window position, starts at final position
rightx = rightf --Current right window position
centerx = centerf --Current center window position
positionID = { 0,0,0 } --Current loaded image
--//reserved Vars
blink_width = 20 --Used for the Continue animation
blink_frames = 3 --Same as above
int_running = false
--//Visibility Switches
gui_visible = true
left_visible = true
right_visible = true
center_visible = true
optionw_visible = false
debug_visible = false
bg_visible = true
text_visible = true
anim_visible = true
--Position wrapper variables
right = "right"
left = "left"
center = "center"
--Object related Variables
object_vis = {}
object_x = {}
object_y = {}
object_gfx = {}
--BG related Variables
BG_change = true --Sets if the BG was changed, in order to redraw.
bg_x = 0
bg_y = 0
bg_image = ""
last_bg = {0,0,""}
last_gui = 0
--Temporal Image
temp_image = false
tempimg = nil

--Functions
function galpsp_charin(charID, position, anim)
	if anim == nil then
		anim = true
	end
	local indexer = indexpos(position)
	if positionID[indexer] == 0 then
		positionID[indexer] = charID
		galpsp_drawchar(charID,position,anim)
		collectgarbage()
		return
	end
	if positionID[indexer] == charID then
		return
	end
	galpsp_out(position)
	positionID[indexer] = charID
	galpsp_drawchar(charID,position,anim)
	collectgarbage()
end

function galpsp_out(posit,exitp)
	if posit == "left" or posit == nil and positionID[1] ~= 0 then
		temp_enable("left")
		if exitp == left or exitp == nil then
		 	leftoutp = leftout
		 	speedp = (speed*-1)
		end
		if exitp == right then
			leftoutp = rightout
			speedp = speed
		end 
		for i=leftx,leftoutp,speedp do
			leftx = i
			galpsp_render()
			screen.waitVblankStart(1)
		end
		temp_disable()
		leftw:clear()
		positionID[1] = 0
		if posit ~= nil then
			return
		end
	end
	if posit == "right" or posit == nil and positionID[2] ~= 0 then
		temp_enable("right")
		if exitp == left then
		 	rightoutp = leftout
		 	speedp = (speed*-1)
		end
		if exitp == right or exitp == nil then
			rightoutp = rightout
			speedp = speed
		end 
		for i=rightx,rightoutp,speedp do
			rightx = i
			galpsp_render()
			screen.waitVblankStart(1)
		end
		temp_disable()
		rightw:clear()
		positionID[2] = 0
		if posit ~= nil then
			return
		end
	end
	if posit == "center"  or posit == nil and positionID[3] ~= 0 then
		temp_enable("center")
		if exitp == left or exitp == nil then
		 	centeroutp = leftout
		 	speedp = (speed*-1)
		end
		if exitp == right then
			centeroutp = rightout
			speedp = speed
		end 
		for i=centerx,centeroutp,speedp do
			centerx = i
			galpsp_render()
			screen.waitVblankStart(1)
		end
		temp_disable()
		centerw:clear()
		positionID[3] = 0
		if posit ~= nil then
			return
		end
	end
	collectgarbage()
end
function galpsp_render(noflip)
	screen:clear()
	bg_changed()
	if temp_image == true then
		screen:blit(0,0,tempimg)
	end
	if bg_visible == true then
		screen:blit(0,0,bg)
	end
	if left_visible == true and positionID[1] ~= 0 then
		screen:blit(leftx,left_y,leftw)
	end
	if right_visible == true and positionID[2] ~= 0 then
		screen:blit(rightx,right_y,rightw)
	end
	if center_visible == true and positionID[3] ~= 0 then
		screen:blit(centerx,center_y,centerw)
	end
	if optionw_visible == true then
		screen:blit(35,174,optionw)
	end
	if text_visible == true then
		screen:blit(textw_x,textw_y,textw)
		screen:blit(namew_x,namew_y,namew)
	end
	if debug_visible == true then
		screen:blit(0,0,debugw)
	end
	if anim_visible == true then
		animw:clear()
		draw_objects()
		screen:blit(0,0,animw)
		screen:blit(440,230,blink)
	end
	--screen.waitVblankStart(1)
	local keyin = Controls.read()
	if keyin:start() and interrupt_start == true and int_running == false then
		int_running = true
		dofile("./Data/Scripts/startint.lua")
		int_running = false
	end
	if interrupt_gen == true and int_running == false then
		int_running = true
		dofile("./Data/Scripts/genint.lua")
		int_running = false
	end
	if noflip == nil or noflip == true then
		screen.flip()
	end
end
function galpsp_drawchar(char,position,anim)
	if position == "left" then
		leftw:clear()
		if anim == true then
			temp_enable("left")
			leftx = leftout
			if type(char) == "string" then
				leftw:blit(0,0,Image.load("./Data/GFX/Characters/"..char..".png"))
			else
				leftw:blit(0,0,char)
			end
			for i=leftout,leftf,speed do
				leftx = i
				galpsp_render()
				--screen.waitVblankStart(1)
			end
			temp_disable()
			return
		else
			leftx = leftf
			if type(char) == "string" then
				leftw:blit(0,0,Image.load("./Data/GFX/Characters/"..char..".png"))
			else
				leftw:blit(0,0,char)
			end
			galpsp_render()
			screen.waitVblankStart(1)
		end
	end
	if position == "right" then
		rightw:clear()
		if anim == true then
			temp_enable("right")
			rightx = -rightout
			if type(char) == "string" then
				rightw:blit(0,0,Image.load("./Data/GFX/Characters/"..char..".png"))
			else
				rightw:blit(0,0,char)
			end
			for i=rightout,rightf,(speed*-1) do
				rightx = i
				galpsp_render()
				--screen.waitVblankStart(1)
			end
			temp_disable()
			return
		else
			rightx = rightf
			if type(char) == "string" then
				rightw:blit(0,0,Image.load("./Data/GFX/Characters/"..char..".png"))
			else
				rightw:blit(0,0,char)
			end
			galpsp_render()
			screen.waitVblankStart(1)
		end
	end
	if position == "center" then
		centerw:clear()
		if anim == true then
			temp_enable("center")
			centerx = -centerout
			if type(char) == "string" then
				centerw:blit(0,0,Image.load("./Data/GFX/Characters/"..char..".png"))
			else
				centerw:blit(0,0,char)
			end
			for i=centerout,centerf,speed do
				centerx = i
				galpsp_render()
				--screen.waitVblankStart(1)
			end
			temp_disable()
			return
		else
			centerx = centerf
			if type(char) == "string" then
				centerw:blit(0,0,Image.load("./Data/GFX/Characters/"..char..".png"))
			else
				centerw:blit(0,0,char)
			end
			galpsp_render()
			screen.waitVblankStart(1)
		end
	end
	screen.waitVblankStart(15)
end
function galpsp_changebg(bgi,bgx,bgy)
	if bgx ~= nil then
		bg_x = bgx
	end
	if bgy ~= nil then
		bg_y = bgy
	end
	bg_image = bgi
	BG_change = true
	galpsp_render()
	collectgarbage()
end
function bg_changed()
	if last_bg[1] ~= bg_x or last_bg[2] ~= bg_y or last_bg[3] ~= bgi or gui_visible ~= last_gui then
		BG_change = true
	end
	if BG_change == true then
		bg:clear()
		if bg_image ~= "" then
			if type(bg_image) == "string" then
				bg:blit(bg_x,bg_y,Image.load("./Data/GFX/BG/"..bg_image..".png"))
			else
				bg:blit(bg_x,bg_y,bg_image)
			end
		end
		if gui_visible == true then
			bg:blit(0,0,gui)
		end
		last_gui = gui_visible
		BG_change = false
		last_bg[1] = bg_x
		last_bg[2] = bg_y
		last_bg[3] = bgi
	end
end
function galpsp_changechar(charID, position)
	local indexer = indexpos(position)
	if positionID[indexer] == 0 then
		return
	end
	if positionID[indexer] == charID then
		return
	end
	galpsp_drawchar(charID,position,false)
	positionID[indexer] = charID
	collectgarbage()
end


function galpsp_clearObj()
	local objn = table.getn(object_vis)
	if objn == 0 then
		return
	end
	for i=1,objn,1 do
		table.remove(object_gfx)
		table.remove(object_vis)
		table.remove(object_x)
		table.remove(object_y)
	end
	collectgarbage()
end

function galpsp_insobj(ox,oy,ogfx,ov,ind)
	local emptys = false
	if ind == nil then
		ind = table.getn(object_vis) + 1
		emptys = true
	end
	if ov ~= nil then
		table.insert(object_vis,ind,ov)
	end
	if ogfx ~= nil then
		if type(ogfx) == "string" then
			table.insert(object_gfx,ind,Image.load("./Data/GFX/Objects/"..ogfx..".png"))
		else
			table.insert(object_gfx,ind,ogfx)
		end
	end
	if ox ~= nil then
		table.insert(object_x,ind,ox)
	end
	if oy ~= nil then
		table.insert(object_y,ind,oy)
	end
	animw:clear()
	galpsp_render()
	if emptys == true then
		return table.getn(object_gfx)
	end
end

function draw_objects()
	animw:clear()
	local objectn = table.getn(object_vis)
	if objectn == 0 then
		return
	end
	for i=1,objectn,1 do
		if object_vis[i] == true then
			animw:blit(object_x[i],object_y[i],object_gfx[i])
		end
	end
end

function galpsp_remobj(indo)
	table.remove(object_vis,indo)
	table.remove(object_gfx,indo)
	table.remove(object_x,indo)
	table.remove(object_y,indo)
	galpsp_render()
	collectgarbage()
end

function temp_create(layert)
	if layert ~= "bg" then
		tempimg:blit(0,0,bg)
		tempimg:blit(0,0,gui)
	end
	if layert ~= "left" and positionID[1] ~= 0 then
		tempimg:blit(leftx,left_y,leftw)
	end
	if layert ~= "right" and positionID[2] ~= 0 then
		tempimg:blit(rightx,right_y,rightw)
	end
	if layert ~= "center" and positionID[3] ~= 0 then
		tempimg:blit(centerx,center_y,centerw)
	end
	if layert ~= "option" then
		tempimg:blit(35,174,optionw)
	end
	if layert ~= "text" then
		tempimg:blit(textw_x,textw_y,textw)
		tempimg:blit(namew_x,namew_y,namew)
	end
	if layert ~= "anim" then
		animw:clear()
		draw_objects()
		tempimg:blit(0,0,animw)
		tempimg:blit(440,230,blink)
	end
end

function temp_enable(layert)
	tempimg = Image.createEmpty(480,272)
	if layert ~= "gui" then
		gui_visible = false
	end
	if layert ~= "left" then
		left_visible = false
	end
	if layert ~= "right" then
		right_visible = false
	end
	if layert ~= "center" then
		center_visible = false
	end
	if layert ~= "optionw" then
		optionw_visible = false
	end
	if layert ~= "bg" then
		bg_visible = false
	end
	if layert ~= "text" then
		text_visible = false
	end
	if layert ~= "anim" then
		anim_visible = false
	end
	temp_image = true
	temp_create(layert)
end

function temp_disable()
	temp_draw = 0
	tempimg = nil
	collectgarbage()
	gui_visible = true
	left_visible = true
	right_visible = true
	center_visible = true
	optionw_visible = false
	debug_visible = false
	bg_visible = true
	text_visible = true
	anim_visible = true
	temp_image = false
end
--EOF